#!/usr/bin/perl
#
# Author: ryu@gpul.org
#
# It's designed for X-Chat
#
# This script is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This script is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You can read the GNU General Public License at http://www.gnu.org

my @secciones; # Formato: "seccion||cantidad"
my @autores;   # Formato: "autor||cantidad"
my $totales=0;

my $base_datos=$ARGV[0] || "";
my $base_puntos=$ARGV[1] || "";

if (!$base_datos || !$base_puntos) {
    print "Sintaxis: estadist.pl <base_datos> <base_puntos>\n";
    exit(1);
}

sub cantidad {
    my $fila=shift;
    $fila=~/.*\|\|(.*)/;
    return $1;
}

sub round {
    my $val=shift;
    my $intval=int($val);
    if ( ($val - $intval) >=0.5) {
	return $intval+1;
    }else{
	return $intval;
    }
}

sub busca {
    my $item=shift;
    my $pos=0;
    for (@_) {
	/(.*)\|\|.*/;
	if ($1=~/^$item$/) {
	    return $pos;
	}else{
	    $pos++;
	}
    }
    return -1;
}

sub inc_seccion {
    my $seccion=shift;
    
    my $pos=busca($seccion,@secciones);
    
    if ($pos==-1) { push @secciones, "$seccion||1"; }
    else {
	$secciones[$pos]=~/$seccion\|\|(.*)/;
	my $cuantos=$1;
	$cuantos++;
	$secciones[$pos]="$seccion||$cuantos";
    }
}

sub inc_autor {
    my $autor=shift;
    
    my $pos=busca($autor,@autores);
    
    if ($pos==-1) { push @autores, "$autor||1"; }
    else {
	$autores[$pos]=~/$autor\|\|(.*)/;
	my $cuantos=$1;
	$cuantos++;
	$autores[$pos]="$autor||$cuantos";
    }
}

open(DATOS,"<$base_datos");

while (<DATOS>) {
    if (/^([^]*)([^]*)[^\*]*\*[^\r\n]*$/) {
	inc_seccion($1);
	inc_autor($2);
    }elsif (/^([^\*]*)\*(.*).$/){
	inc_seccion("Sin Seccin");
	inc_autor("Sin Autor");
    }
    $totales++;
}

close(DATOS);

my @puntos; # Formato: "nick||cantidad"

sub add_puntos {
    my $nick=shift;
    my $pnt=shift;
    
    my $pos=busca($nick,@puntos);
    if ($pos==-1) { push @puntos, "$nick||$pnt"; }
    else {
	$puntos[$pos]=~/$seccion\|\|(.*)/;
	my $cuantos=$1;
	$cuantos+=$pnt;
	$puntos[$pos]="$nick||$cuantos";
    }
}

open(PUNTOS,"<$base_puntos");
my $pos=0;
my $partidas=0;
while(<PUNTOS>) {
    if (/^\* (.*)/) {
	$pos=0;
	$partidas++;
    }else{
	/^([^:]*):(.*)$/;
	add_puntos($1,5-$pos);
	$pos++;
    }
}
close(PUNTOS);

print "<html><body bgcolor=\"#d6c2c2\" text=\"#573232\">";
print "<center><font size=+2><b>#Trivial-Linux</b></font></center><br>\n";
print "<br><b>ndice</b>\n";
print "<ul>\n";
print "<li><a href=\"#bd\">Estadsticas de la base de datos</a>\n";
print "<li><a href=\"#compet\">Estadsticas de las competiciones</a>\n";
print "</ul>\n";

print "<a name=\"bd\">\n";
print "<p><b>1</b>. Estadsticas de la base de datos\n";
print "</a>\n";
print "<p>$totales preguntas.</p>\n";
print "<table border=\"0\" cellpadding=\"10\" width=\"100%\"><tr><td valign=\"top\">\n";
print "<p> Numero de preguntas por temas:<br>\n";
print "<table border=\"0\" width=\"100%\" cellpadding=\"2\" cellspacing=\"0\" bgcolor=\"#ba8686\">\n";
my $headFont="color=\"#daa6a6\"";
print "<tr bgcolor=\"876262\"><td><font $headFont>Tema</font></td><td align=\"right\"><font $headFont>Cantidad</font></td><td align=\"right\"><font $headFont>Porcentaje de preguntas</font></td></tr>\n";
for (sort {cantidad($b) <=> cantidad($a)} @secciones) {
    /(.*)\|\|(.*)/;
    my $porcent=round(($2*1000)/$totales)/10;
    printf "<tr><td>$1</td><td align=\"right\">$2</td><td align=\"right\"> <table><tr><td>%3.1f %</td><td width=\"$porcent\" bgcolor=\"#dd5555\"\>&nbsp</td></tr></table></td></tr>\n", $porcent;
}
print "</table>\n";
my $numsec=$#secciones+1;
print "<br>$numsec secciones.\n";
print "</td><td valign=\"top\">\n";
print "<p> Numero de preguntas por autores:<br>\n";
print "<table border=\"0\" width=\"100%\" cellpadding=\"2\" cellspacing=\"0\" bgcolor=\"#ba8686\">\n";
print "<tr bgcolor=\"876262\"><td><font $headFont>Autor</font></td><td align=\"right\"><font $headFont>Cantidad</font></td><td align=\"right\"><font $headFont>Porcentaje de preguntas</font></td></tr>\n";
for (sort {cantidad($b) <=> cantidad($a)} @autores) {
    /(.*)\|\|(.*)/;
    my $porcent=round(($2*1000)/$totales)/10;
    printf "<tr><td>$1</td><td align=\"right\">$2</td><td align=\"right\"><table><tr><td>%3.1f %</td><td width=\"$porcent\" bgcolor=\"#dd5555\"\>&nbsp</td></tr></table></td></tr>\n", $porcent;
}
print "</table>\n";
my $numaut=$#autores+1;
print "<br>$numaut autores.\n";
print "</td></tr></table>\n";


print "<a name=\"compet\">\n";
print "<p><b>2</b>. Estadsticas de las competiciones\n";
print "</a>\n";
print "<p>$partidas partidas.</p>\n";

if (@puntos) {
    print "<table border=\"0\" width=\"50%\" cellpadding=\"3\" cellspacing=\"0\" bgcolor=\"#ba8686\">\n";
    print "<tr bgcolor=\"876262\"><td><font $headFont>Nick</font></td><td align=\"right\"><font $headFont>Puntos</font></td></tr>\n";
    for (sort {cantidad($b) <=> cantidad($a)} @puntos) {
	/(.*)\|\|(.*)/;
	printf "<tr><td>$1</td><td align=\"right\">$2</td></tr>\n";
    }
    print "</table>\n";
}
print "</body></html>";
